/*******************************************************
FEED PARSER

Original code by Ryan Parman
(c) 2004, Ryan Parman, http://www.skyzyx.com

Greatly rewritten by Sudhakar Circa Oct 2004.

Distributed according to SkyGPL 2.1, http://www.skyzyx.com/license/
*******************************************************/

/*******************************************************
INITIALIZE GLOBAL VARIABLES
*******************************************************/
var gecko= (document.implementation && document.implementation.createDocument) ? true:false;

/*******************************************************
WHITESPACE CLEANER FOR MOZILLA
By Alex Vincent, http://weblogs.mozillazine.org/weirdal/
*******************************************************/
const notWhitespace = /\S/

function cleanWhitespace(node) {
    for (var x = 0; x < node.childNodes.length; x++) {
	var childNode = node.childNodes[x];
	if ((childNode.nodeType == 3)&&(!notWhitespace.test(childNode.nodeValue))) {
	    node.removeChild(node.childNodes[x]);
	    x--;
	}
	if (childNode.nodeType == 1) { cleanWhitespace(childNode); }
    }
}

/*******************************************************
FIX MIMETYPES/DOCUMENT HEADERS FOR GECKO
By Vladdy, http://www.klproductions.com/
*******************************************************/
function fixXmlMimeType(filename)
{
    oxmlhttp = null;

    try {
	oxmlhttp = new XMLHttpRequest();
	oxmlhttp.overrideMimeType("application/xml");
    } catch(e) {
	try { oxmlhttp = new ActiveXObject("Msxml2.XMLHTTP"); }
	catch(e) { return null; }
    }

    if (!oxmlhttp) return null;

    try {
	// Gecko must have this security priviledge enabled.
	if (gecko) netscape.security.PrivilegeManager.enablePrivilege("UniversalBrowserRead");

	oxmlhttp.open("GET", filename, false);
	oxmlhttp.send(null);
    } catch(e) { return null; }

    return oxmlhttp.responseXML;
}


/*******************************************************
"FAMILIAL-HEIRARCHY" NODE DEPTH FIX (WITH CASE-INSENSITIVITY)
By David "Liorean" Andersson, http://liorean.web-graphics.com/
*******************************************************/
function getChildrenByTagName(oParent, sTagName)
{
    var i=0, aResults=[], oTemp;
    var reTagName = new RegExp('^'+sTagName+'$', 'i');

    while ((oTemp=oParent.childNodes.item(i++))) {
	if (reTagName.test(oTemp.nodeName)) aResults.push(oTemp);
    }

    return aResults;
}


// Utility function to get a required item from the feed
function getRequiredItem(xmlChunk,requiredItem) {
    return getChildrenByTagName(xmlChunk,requiredItem)[0].firstChild.data;
}

// Utility function to get an optional item from the feed
function getOptionalItem(xmlChunk,optionalItem) {
    try {
	return getChildrenByTagName(xmlChunk, optionalItem)[0].firstChild.data;
    } catch (e) { return false; }
}


/*******************************************************
CREATE PARSING OBJECT
*******************************************************/
function FeedParser(rssFeedUrl)
{
    var xmlFeed=false;
    if (xmlFeed === false) xmlFeed = fixXmlMimeType(rssFeedUrl);

    // Feed Parser core data
    this.fp = new Object();
    this.fp.author = "Ryan Parman"
    this.fp.contributors = "David Andersson, Vladdy, and Alex Vincent"
    this.fp.copyright = "2004";
    this.fp.name = "Feed Parser"
    this.fp.url = "http://www.skyzyx.com/projects/feedparser/";
    this.fp.version = "0.9.0";
    this.fp.build = "20041004";

    // RSS Feed URL
    this.feedUrl = rssFeedUrl;

    // Gecko must have this security priviledge enabled.
    if (gecko) netscape.security.PrivilegeManager.enablePrivilege("UniversalBrowserRead");

    // Set this.type, this.type[Format], and clean whitespace from all XML nodes.
    this.typeRSS = false; this.typeRDF = false; this.typeAtom = false;
    if (xmlFeed.getElementsByTagName('rss')[0]) {
	this.type = "RSS"; this.typeRSS = true; 
	cleanWhitespace(xmlFeed.getElementsByTagName('rss')[0]); 
    } else if (xmlFeed.getElementsByTagName('RDF')[0]) {
	this.type = "RDF"; this.typeRDF = true; 
	cleanWhitespace(xmlFeed.getElementsByTagName('RDF')[0]); 
    } else if (xmlFeed.getElementsByTagName('feed')[0]) {
	this.type = "Atom"; this.typeAtom = true; 
	cleanWhitespace(xmlFeed.getElementsByTagName('feed')[0]); 
    } else this.type = false;

    this.version = (this.typeRSS) ? xmlFeed.getElementsByTagName('rss')[0].getAttribute('version') : (this.typeRDF) ? "1.0" : (this.typeAtom) ? xmlFeed.getElementsByTagName('feed')[0].getAttribute('version') : "0.0";

    var MasterElem;
    if (this.typeRSS) {
	MasterElem = xmlFeed.getElementsByTagName('rss')[0];
    } else if (this.typeRDF) {
	MasterElem = xmlFeed.getElementsByTagName('RDF')[0];
    } else if (this.typeAtom) {
	MasterElem = xmlFeed.getElementsByTagName('feed')[0];
    }


    /*******************************************************
    Begin parsing the RSS 0.9x/2.0 specification.
    *******************************************************/
    if (this.typeRSS) {
	// Master RSS Element
	var RSSElem = xmlFeed.getElementsByTagName('rss')[0];

	// Channel
	var RSSChannel = getChildrenByTagName(RSSElem, 'channel')[0];
	this.channel = new Object();
	this.channel.readTag = RSSChannel;

	// Required tags.  Parsing will fail without these.
	this.channel.title	    = getRequiredItem(RSSChannel, 'title');

	// Optional feed tags with decent default values
	this.channel.link	    = getOptionalItem(RSSChannel,'link') ? getOptionalItem(RSSChannel,'link') : this.feedUrl;
	this.channel.description    = getOptionalItem(RSSChannel,'description') ? getOptionalItem(RSSChannel,'description') : 'No description available';

	// Optional feed tags with 'false' default value
	this.channel.category	    = getOptionalItem(RSSChannel,'category');

	// Different ways of specifying copyright
	this.channel.dcRights	    = getOptionalItem(RSSChannel,'dc:rights');
	this.channel.copyright	    = getOptionalItem(RSSChannel,'copyright');
	this.channel.ccLicense	    = getOptionalItem(RSSChannel,'ccLicense');

	// Consolidate copyright into this.channel.copyright
	if (this.channel.dcRights) {
	    this.channel.copyright = this.channel.dcRights;
	} else if (this.channel.ccLicense) {
	    this.channel.copyright = this.channel.ccLicense;
	} else if (! this.channel.copyright) {
	    this.channel.copyright = false;
	}

	this.channel.generator	    = getOptionalItem(RSSChannel,'generator');
	this.channel.language	    = getOptionalItem(RSSChannel,'language');

	// Different ways of specifying dates
	this.channel.pubDate	    = getOptionalItem(RSSChannel,'pubDate');
	this.channel.lastBuildDate  = getOptionalItem(RSSChannel,'lastBuildDate');
	this.channel.dcDate	    = getOptionalItem(RSSChannel,'dc:Date');

	// Consolidate date into this.channel.pubDate
	if (this.channel.dcDate) {
	    this.channel.pubDate = this.channel.dcDate;
	} else if (this.channel.lastBuildDate) {
	    this.channel.pubDate = this.channel.lastBuildDate;
	} else if (! this.channel.pubDate) {
	    this.channel.pubDate = false;
	}

	var RSSImage = (getChildrenByTagName(RSSChannel, 'image')[0]) ? getChildrenByTagName(RSSChannel, 'image')[0] : false;
	if (RSSImage) {
	    this.channel.image		= new Object();
	    this.channel.image.url	= getOptionalItem(RSSImage,'url');
	    this.channel.image.link	= getOptionalItem(RSSImage,'link');
	    this.channel.image.title	= getOptionalItem(RSSImage,'title');
	    this.channel.image.width	= getOptionalItem(RSSImage,'width');
	    this.channel.image.height	= getOptionalItem(RSSImage,'height');
	} else {
	    this.channel.image = false;
	}

	// <item>, required tag set.  Multiple are allowed, so make them all parsable.
	this.channel.item = new Array();
	for (var x=0; x<getChildrenByTagName(RSSChannel, 'item').length; x++) {
	    this.channel.item[x] = getChildrenByTagName(RSSChannel, 'item')[x];
	}

	for (var y=0; y<this.channel.item.length; y++) {
	    var RSSItem = (getChildrenByTagName(RSSChannel, 'item')[y]) ? getChildrenByTagName(RSSChannel, 'item')[y] : false;
	    this.channel.item[y] = new Object();

	    this.channel.item[y].link		=   getOptionalItem(RSSItem,'link');
	    this.channel.item[y].guid		=   getOptionalItem(RSSItem,'guid');

	    // Give more preference to link than guid
	    if (this.channel.item[y].guid && ! this.channel.item[y].link) {
		this.channel.item[y].link = this.channel.item[y].guid;
	    }

	    this.channel.item[y].title		=   getOptionalItem(RSSItem,'title');
	    this.channel.item[y].description	=   getOptionalItem(RSSItem,'description');

	    // TODO Enclosures
	    /*
	    this.channel.item[y].enclosureElement =   RSSItem.getElementsByTagName('enclosure');
	    if (this.channel.item[y].enclosureElement) {
		alert(this.channel.item[y].enclosureElement);
		this.channel.item[y].enclosureURL = this.channel.item[y].enclosureElement[0].getAttribute('url')
	    }
	    */

	    // Different ways of specifying the author
	    this.channel.item[y].author		=   getOptionalItem(RSSItem,'author');
	    this.channel.item[y].dcCreator	=   getOptionalItem(RSSItem,'dc:creator');
	    this.channel.item[y].dcContributor	=   getOptionalItem(RSSItem,'contributor');

	    // Consolidate into this.channel.item[y].author
	    if (this.channel.item[y].dcCreator) {
		this.channel.item[y].author = this.channel.item[y].dcCreator;
	    } else if (this.channel.item[y].dcControbutor) {
		this.channel.item[y].author = this.channel.item[y].dcContributor;
	    } else if (! this.channel.item[y].author) {
		this.channel.item[y].author = false;
	    }

	    // Different ways of specifying the subject
	    this.channel.item[y].category	=   getOptionalItem(RSSItem,'category');
	    this.channel.item[y].dcSubject	=   getOptionalItem(RSSItem,'dc:subject');

	    // Consolidate into this.channel.item[y].category
	    if (this.channel.item[y].dcSubject) {
		this.channel.item[y].category = this.channel.item[y].dcSubject;
	    } else if (! this.channel.item[y].category) {
		this.channel.item[y].category = false;
	    }

	    // Different ways of specifying dates
	    this.channel.item[y].pubDate	=   getOptionalItem(RSSItem,'pubDate');
	    this.channel.item[y].dcDate		=   getOptionalItem(RSSItem,'dc:date');

	    if (this.channel.item[y].dcDate) {
		this.channel.item[y].pubDate = this.channel.item[y].dcDate;
	    } else if (! this.channel.item[y].pubDate) {
		this.channel.item[y].pubDate = false;
	    }
	}
    } else if (this.typeRDF) {
	/*******************************************************
	Begin parsing the RSS 1.0 (RDF) specification.
	*******************************************************/
	// Master RDF Element
	var RDFElem = xmlFeed.getElementsByTagName('RDF')[0];

	// Channel
	var RSSChannel = getChildrenByTagName(RDFElem, 'channel')[0]
	this.channel = new Object();
	this.readTag = RDFElem;

	// Required tags.  Parsing will fail without these.
	this.channel.title	    = getRequiredItem(RSSChannel, 'title');

	// Optional tags with sensible default values
	this.channel.link	    = getOptionalItem(RSSChannel, 'link') ? getOptionalItem(RSSChannel, 'link') : this.feedUrl;

	// Different ways of specifying description
	this.channel.description    = getOptionalItem(RSSChannel, 'description') ? getOptionalItem(RSSChannel, 'description') : 'No description available';
	this.channel_rdfAbout = (getChildrenByTagName(RDFElem, 'channel')[0]) ? getChildrenByTagName(RDFElem, 'channel')[0].getAttributeNS(this.nsRDF, 'about') : false;

	// Consolidate into this.channel.description
	if (this.channel_rdfAbout) {
	    this.channel.description = this.channel_rdfAbout;
	} else if (! this.channel.description) {
	    this.channel.description = false;
	}

	// Different ways of specifying copyright
	this.channel.dcRights	    = getOptionalItem(RSSChannel, 'dc:rights');
	this.channel.copyright	    = getOptionalItem(RSSChannel, 'copyright');
	this.channel.ccLicense	    = getOptionalItem(RSSChannel, 'ccLicense');

	// Consolidate copyright into this.channel.copyright
	if (this.channel.dcRights) {
	    this.channel.copyright = this.channel.dcRights;
	} else if (this.channel.ccLicense) {
	    this.channel.copyright = this.channel.ccLicense;
	} else if (! this.channel.copyright) {
	    this.channel.copyright = false;
	}

	// <items> (note the "s").
	var RSSItems = (getChildrenByTagName(RSSChannel, 'items')[0]) ? getChildrenByTagName(RSSChannel, 'items')[0] : false;
	this.channel.items = new Object();
	var RDFSeq = (RSSItems) ? RSSItems.getElementsByTagNameNS(this.nsRDF, 'Seq')[0] : false;
	var RDFSeqLi = (RDFSeq) ? RDFSeq.getElementsByTagNameNS(this.nsRDF, 'li') : false;
	this.channel.items.rdfSeq = new Object();

	// <rdf:li>, required tag set.  Multiple are allowed, so make them all parsable.
	this.channel.items.rdfSeq.li = new Array();
	this.channel.items.rdfSeq.li_rdfResource = new Array();
	for (var x=0; x<RDFSeqLi.length; x++) {
	    this.channel.items.rdfSeq.li[x] = RDFSeqLi[x];
	    this.channel.items.rdfSeq.li_rdfResource[x] = RDFSeqLi[x].getAttributeNS(this.nsRDF, 'resource');
	}

	var RSSImage = (getChildrenByTagName(RDFElem, 'image')[0]) ? getChildrenByTagName(RDFElem, 'image')[0] : false;
	if (RSSImage) {
	    this.image		= new Object();
	    this.image.url	= getOptionalItem(RSSImage,'url');
	    this.image.link	= getOptionalItem(RSSImage, 'link');
	    this.image.title	= getOptionalItem(RSSImage, 'title');
	}

	// <item>, required tag set.  Multiple are allowed, so make them all parsable.
	this.item = new Array();
	this.item_rdfAbout = new Array();
	for (var x=0; x<getChildrenByTagName(RDFElem, 'item').length; x++) {
	    this.item[x] = getChildrenByTagName(RDFElem, 'item')[x];
	    this.item_rdfAbout[x] = getChildrenByTagName(RDFElem, 'item')[x].getAttributeNS(this.nsRDF, 'about');
	}

	this.channel.item = new Array();
	for (var y=0; y<this.item.length; y++) {
	    var RSSItem = (getChildrenByTagName(RDFElem, 'item')[y]) ? getChildrenByTagName(RDFElem, 'item')[y] : false;

	    this.channel.item[y] = new Object();

	    // this.item[y].link = (getChildrenByTagName(RSSItem, 'link')[0]) ? getChildrenByTagName(RSSItem, 'link')[0].firstChild.data : false;
	    this.channel.item[y].title		= getOptionalItem(RSSItem, 'title');

	    // Different ways of specifying links
	    this.channel.item[y].link		= getOptionalItem(RSSItem, 'link');
	    this.channel.item[y].guid		= getOptionalItem(RSSItem, 'guid');

	    // Consolidate into this.channel.item[y].link
	    if (this.channel.item[y].guid) {
		this.channel.item[y].link = this.channel.item[y].guid;
	    }

	    this.channel.item[y].description	= getOptionalItem(RSSItem, 'description');

	    this.channel.item[y].dcSubject	= getOptionalItem(RSSItem, 'dc:subject');

	    if (this.channel.item[y].dcSubject) {
		this.channel.item[y].category = this.channel.item[y].dcSubject;
	    } else {
		this.channel.item[y].category = false;
	    }

	    // Different ways of specifying author
	    this.channel.item[y].dcCreator	= getOptionalItem(RSSItem, 'dc:creator');
	    this.channel.item[y].dcContributor	= getOptionalItem(RSSItem, 'dc:contributor');

	    // Consolidate into this.channel.item[y].author
	    if (this.channel.item[y].dcCreator) {
		this.channel.item[y].author = this.channel.item[y].dcCreator;
	    } else if (this.channel.item[y].dcContributor) {
		this.channel.item[y].author = this.channel.item[y].dcContributor;
	    } else {
		this.channel.item[y].author = false;
	    }

	    this.channel.item[y].dcDate		= getOptionalItem(RSSItem, 'dc:date');

	    if (this.channel.item[y].dcDate) {
		var date = this.channel.item[y].dcDate.substr(0,10);
		var time = this.channel.item[y].dcDate.substr(11,8);
		var tz = this.channel.item[y].dcDate.substr(19);

		this.channel.item[y].pubDate = date + " " + time + " GMT" + tz;
	    }

	}
    } else if (this.typeAtom) {
	/*******************************************************
	Begin parsing the Atom 0.3 specification.
	*******************************************************/
	// Master Feed Element
	var feedElem = xmlFeed.getElementsByTagName('feed')[0];
	this.xmlLang = feedElem.getAttribute('xml:lang');

	// Read any tag...
	this.readTag = feedElem;

	this.channel = new Object();

	this.author	= new Object();
	var feedAuthor	= getChildrenByTagName(feedElem, 'author')[0];

	if (feedAuthor) {
	    this.author.email	= getOptionalItem(feedAuthor,'email');
	    this.author.name	= getOptionalItem(feedAuthor,'name');
	    this.author.url	= getOptionalItem(feedAuthor,'url');
	}

	this.channel.title	    = getRequiredItem(feedElem,'title');
	this.channel.description	    = getOptionalItem(feedElem,'tagline') ? getOptionalItem(feedElem,'tagline') : '';

	this.channel.copyright	    = getOptionalItem(feedElem,'copyright');

	this.channel.info	    = getOptionalItem(feedElem,'info');
	this.channel.pubDate	    = getOptionalItem(feedElem,'modified');
	this.channel.generator	    = getOptionalItem(feedElem,'generator');

	this.link = getChildrenByTagName(feedElem, 'link')[0];
	if (this.link) {
	    this.link_href = this.link.getAttribute('href'); // Required
	    this.link_rel = this.link.getAttribute('rel'); // Required
	    this.link_title = (this.link.getAttribute('title')) ? this.link.getAttribute('title') : false; // Optional
	    this.link_type = (this.link.getAttribute('type')) ? this.link.getAttribute('type') : false; // Optional

	    this.channel.link = this.link_href;
	}

	// <entry> -- Multiple are allowed, so make them all parsable.
	this.channel.item = new Array();
	for (var x=0; x<getChildrenByTagName(feedElem, 'entry').length; x++) {
	    this.channel.item[x] = new Object();

	    this.channel.item[x] = getChildrenByTagName(feedElem, 'entry')[x];
	}

	for (var y=0; y<this.channel.item.length; y++) {
	    var feedEntry = (getChildrenByTagName(feedElem, 'entry')[y]) ? getChildrenByTagName(feedElem, 'entry')[y] : false;

	    this.channel.item[y] = new Object();
	    if (feedEntry) {
		this.channel.item[y].contributor   = getOptionalItem(feedEntry,'contributor');
		this.channel.item[y].created	    = getOptionalItem(feedEntry,'created');
		this.channel.item[y].id	    = getOptionalItem(feedEntry,'id');
		this.channel.item[y].issued	    = getOptionalItem(feedEntry,'issued');
		this.channel.item[y].pubDate	    = getOptionalItem(feedEntry,'modified');
		this.channel.item[y].title	    = getOptionalItem(feedEntry,'title');
	    }

	    this.channel.item[y].auth = new Object();
	    var entryAuthor = getChildrenByTagName(feedEntry, 'author')[0];
	    if (entryAuthor) {
		this.channel.item[y].auth.email  = getOptionalItem(entryAuthor,'email');
		this.channel.item[y].auth.name   = getOptionalItem(entryAuthor,'name');
		this.channel.item[y].auth.url    = getOptionalItem(entryAuthor,'url');

		this.channel.item[y].author = this.channel.item[y].auth.name;
	    }

	    if (this.channel.item[y].contributor) {
		this.channel.item[y].author = this.channel.item[y].contributor;
	    }

	    try {
		this.channel.item[y].content = (getChildrenByTagName(feedEntry, 'content')[0]) ? getChildrenByTagName(feedEntry, 'content')[0].firstChild.nodeValue : false;
		this.channel.item[y].content_mode = (getChildrenByTagName(feedEntry, 'content')[0].getAttribute('mode')) ? getChildrenByTagName(feedEntry, 'content')[0].getAttribute('mode') : false;
		this.channel.item[y].content_type = (getChildrenByTagName(feedEntry, 'content')[0].getAttribute('type')) ? getChildrenByTagName(feedEntry, 'content')[0].getAttribute('type') : false;
		this.channel.item[y].content_xmlBase = (getChildrenByTagName(feedEntry, 'content')[0].getAttribute('xml:base')) ? getChildrenByTagName(feedEntry, 'content')[0].getAttribute('xml:base') : false;
		this.channel.item[y].content_xmlLang = (getChildrenByTagName(feedEntry, 'content')[0].getAttribute('xml:lang')) ? getChildrenByTagName(feedEntry, 'content')[0].getAttribute('xml:lang') : false;
	    } catch (e) {}

	    this.channel.item[y].lnk = getChildrenByTagName(feedEntry, 'link')[0];
	    this.channel.item[y].link_href = this.channel.item[y].lnk.getAttribute('href'); // Required
	    this.channel.item[y].link_rel = this.channel.item[y].lnk.getAttribute('rel'); // Required
	    this.channel.item[y].link_title = (this.channel.item[y].lnk.getAttribute('title')) ? this.channel.item[y].lnk.getAttribute('title') : false; // Optional
	    this.channel.item[y].link_type = (this.channel.item[y].lnk.getAttribute('type')) ? this.channel.item[y].lnk.getAttribute('type') : false; // Optional

	    this.channel.item[y].link = this.channel.item[y].link_href;

	    try {
		this.channel.item[y].summary = getChildrenByTagName(feedEntry, 'summary')[0].firstChild.data;
		this.channel.item[y].summary_type = getChildrenByTagName(feedEntry, 'summary')[0].getAttribute('type');
	    } catch (e) {
		this.channel.item[y].summary = false;
		this.channel.item[y].summary_type = false;
	    }

	    if (this.channel.item[y].summary) {
		alert('Summary is ' + this.channel.item[y].summary);
		this.channel.item[y].description = this.channel.item[y].summary;
	    } else if (this.channel.item[y].content) {
		this.channel.item[y].description = this.channel.item[y].content;
	    } else {
		this.channel.item[y].description = false;
	    }

	    this.channel.item[y].issued = getOptionalItem(feedEntry, 'issued');

	    if (this.channel.item[y].issued) {
		var date = this.channel.item[y].issued.substr(0,10);
		var time = this.channel.item[y].issued.substr(11,8);
		var tz = this.channel.item[y].issued.substr(19);

		this.channel.item[y].pubDate = date + " " + time + " GMT" + tz;
	    }
	}
    }

    /*******************************************************
    Begin parsing the RSS 1.0 Dublin Core Metadata Element Set.
    Allowed in <channel>, <item>, <image>, and <textinput> elements.
    *******************************************************/
    if (this.typeRSS || this.typeRDF) {
	var DCChannel = getChildrenByTagName(MasterElem, 'channel')[0];
	// Add parsing for immediate subelements of <channel>
	this.channel.dcContributor = getOptionalItem(DCChannel,'contributor');
	this.channel.dcCoverage = getOptionalItem(DCChannel,'coverage');
	this.channel.dcCreator = getOptionalItem(DCChannel,'dc:creator');
	this.channel.dcDate = getOptionalItem(DCChannel,'date');
	this.channel.dcLanguage = getOptionalItem(DCChannel,'dc:language');
	this.channel.dcDescription = getOptionalItem(DCChannel,'description') ? getOptionalItem(DCChannel,'description') : 'No description available';
	this.channel.dcRights = getOptionalItem(DCChannel,'dc:rights');
	this.channel.dcSubject = getOptionalItem(DCChannel,'subject');

	this.channel.dcTitle = getOptionalItem(DCChannel,'title');
    }
}
